# Rules for all ELT machines.
class eltgeneral {

	# A few users and groups needed by default in ELT
	user { "eltmgr":
		ensure => "present",
		uid => "8267",
		gid => "39542",
		home => "/home/eltmgr",
		managehome => true,
	}
	# Change pass only if not present, default password => "HOuJxJbENTBs2" (pass4u)
	exec {	"/usr/sbin/usermod -p 'HOuJxJbENTBs2' eltmgr":
		onlyif => "/bin/egrep -q '^eltmgr:[*!]' /etc/shadow",
		require => User["eltmgr"],
	}
	# ELTDEV-230 Add user eltdev to group video
	user { "eltdev":
		ensure => "present",
		uid => "6173",
		gid => "39542",
		groups => [ "video", "systemd-journal" ],
		home => "/home/eltdev",
		managehome => true,
	}
	# Change pass only if not present, default password => "x/PCPZWO2m8so" (2Garch1ng)
	exec {	"/usr/sbin/usermod -p 'x/PCPZWO2m8so' eltdev":
		onlyif => "/bin/egrep -q '^eltdev:[*!]' /etc/shadow",
		require => User["eltdev"],
	}


        user { "jenkins":
                ensure => "present",
                uid => "5916",
                gid => "jenkins",
#                home => "/home/jenkins",
                password => '$6$HNs1Z4Sj$0azE97yOqLkrUnITnGclrJ0yXOJPrukkm9Rihc6yD5UhpHpbG0JJ4SHmaj2i3ZZnpwQHmNDq.E8hTjoYLLOxY0',
                comment => "local Jenkins account",
                require => Group["jenkins"],
        }
        group { "jenkins":
                ensure => "present",
                gid => "5916",
        }
        group { "realtime":
                ensure => "present",
                gid => "71",
        }
	group { "elt":
		ensure => "present",
		gid => "39542",
	}


	# Disable default firewalls
	service { "firewalld":
		ensure => "stopped",
		enable => "false",
	}

	# ELTDEV-131 Disable default abrtd
	service { "abrtd":
		ensure => "stopped",
		enable => "false",
	}

	# Disable SELinux
	augeas { "selinux":
                context => "/files/etc/sysconfig/selinux",
                changes => [
                        "set SELINUX disabled",
                 ],
                 notify => Exec["selinux_immediate_stop"],
        }

        exec { "selinux_immediate_stop":
                command => "/usr/sbin/setenforce 0",
                refreshonly => "true",
        }


	# Add ESO-CentOS repository
	file { "ESO-CentOS.repo":
		path => "/etc/yum.repos.d/ESO-CentOS.repo",
		ensure => "present",
                owner => "root",
                group => "root",
                mode => "0644",
		source => "puppet:///modules/eltgeneral/ESO-CentOS.repo",
	}

	yumrepo { "ESO-BaseOS":
		descr => "ESO CentOS-\$releasever - Base",
		enabled => "1",
		require => File["ESO-CentOS.repo"],
	}

	# Add ESO-ELT repository

	yumrepo { "ESO-ELT-CentOS-8":
		descr => "ESO repository for ELT Linux DevEnv",
		baseurl => "ftp://ftp.eso.org/pub/elt/ELTREPO/CentOS-8",
		ensure => "present",
		enabled => "1",
		gpgcheck => "0",
		metadata_expire => "1m",
	}

	# Add ESO-Beta repository
	yumrepo { "ESO-Beta-CentOS-8":
		descr => "ESO Beta repository for ELT Linux DevEnv",
		baseurl => "ftp://ftp.eso.org/pub/elt/ELTREPO/Beta-CentOS-8",
		ensure => "present",
		enabled => "0",
		gpgcheck => "0",
		metadata_expire => "1m",
	}

	# Add ESO-ELT experimental repository, but disabled by default
	yumrepo { "ESO-ELT-CentOS-8x-Experimental":
		descr => "ESO Experimental repository for ELT Linux DevEnv",
		baseurl => "ftp://ftp.eso.org/pub/elt/ELTREPO/CentOS-8x-Experimental",
		ensure => "present",
		enabled => "0",
		gpgcheck => "0",
		metadata_expire => "1m",
	}

	# ELTDEV-118 Add ESO Local repository, but disabled by default
	yumrepo { "ESO-Local-CentOS-8":
		descr => "ESO local repository for ELT Linux DevEnv",
		baseurl => "http://fenice.hq.eso.org/REPO/Local-CentOS-8",
		ensure => "present",
		enabled => "0",
		gpgcheck => "0",
		metadata_expire => "1m",
	}

	# Disable default ones, so we get all from our site
	yumrepo { "BaseOS" :
		descr => "CentOS-\$releasever - Base",
		enabled => "0",
	}
	yumrepo { "AppStream" :
		descr => "CentOS-\$releasever - AppSream",
		enabled => "0",
	}
	yumrepo { "updates" :
		descr => "CentOS-\$releasever - Updates",
		enabled => "0",
	}
	yumrepo { "extras" :
		descr => "CentOS-\$releasever - Extras",
		enabled => "0",
	}
	yumrepo { "centosplus" :
		descr => "CentOS-\$releasever - Plus",
		enabled => "0",
	}
	yumrepo { "epel" :
		name => "epel",
		descr => "Extra Packages for Enterprise Linux 8 - \$basearch",
		enabled => "0",
	}
	yumrepo { "epel-modular" :
		name => "epel-modular",
		descr => "Extra Packages for Enterprise Linux Modular \$releasever - \$basearch",
		enabled => "0",
	}

	# Ensure local time zone is UTC (ELTMGR-191)
	file { "/etc/localtime":
		ensure => "link",
		target => "../usr/share/zoneinfo/UTC",
	} 

	# Sudo rules for unattended Jenkins upgrades
	file { "/etc/sudoers.d/jenkins":
		ensure => "present",
                owner => "root",
                group => "root",
                mode => "0640",
		source => "puppet:///modules/eltgeneral/jenkins-sudoers",
	}
	file_line { "sudoers_jenkins_puppet_2":
		path => "/etc/sudoers",
		line => "#Defaults    requiretty",
		match => "Defaults    requiretty"
	}

	# ELTDEV-112 Sudo rules for eltdev account
	file { "/etc/sudoers.d/eltdev":
		ensure => "present",
                owner => "root",
                group => "root",
                mode => "0640",
		source => "puppet:///modules/eltgeneral/eltdev-sudoers",
	}

	# File containing the role of the machine
	file { "/etc/profile.d/eltdev.sh":
		ensure => "file",
		owner => "root",
		group => "root",
		mode => "0755",
		replace => "false",
		source => "puppet:///modules/eltgeneral/eltdev.sh",
	}
        file_line { "elt-role-variable":
                path => "/etc/profile.d/eltdev.sh",
                line => "export ELT_ROLE=MINIMAL",
                match => "export ELT_ROLE=",
                require => File["/etc/profile.d/eltdev.sh"],
        }

        file_line { "elt-dm-variable":
                path => "/etc/profile.d/eltdev.sh",
        	line => "export ELT_DM=$elt_dm",
                match => "export ELT_DM=",
                require => File["/etc/profile.d/eltdev.sh"],
        }

        file_line { "elt-realtime":
                path => "/etc/profile.d/eltdev.sh",
        	line => "export ELT_REALTIME=$elt_realtime",
                match => "export ELT_REALTIME=",
                require => File["/etc/profile.d/eltdev.sh"],
        }

        file_line { "elt-release":
                path => "/etc/profile.d/eltdev.sh",
                line => "export ELT_RELEASE=$elt_release",
                match => "export ELT_RELEASE=",
                require => File["/etc/profile.d/eltdev.sh"],
        }

	# Doxypypy support script
	file { "/usr/local/bin/py_filter":
		ensure => "present",
		owner => "root",
		group => "root",
		mode => "0755",
		source => "puppet:///modules/eltgeneral/py_filter",
	}


        # /etc/motd as by kickstart
        case $facts['puppetversion'] {
            /^3./:   { $motd_os_info = "${facts['operatingsystem']} Linux ${facts['operatingsystemrelease']} (${facts['architecture']})" }
            default: { $motd_os_info = "${facts['os']['name']} Linux ${facts['os']['release']['major']}.${facts['os']['release']['minor']} (${facts['os']['architecture']})" }
        }
        $puprpm = $facts['puppetrpmver'] 
        $motd_default = @("EOT")

                                             ${motd_os_info}
                                          #####################################
                                             SOFTWARE RELEASE DevEnv $puprpm
                                          #####################################

                | EOT
        file { "/etc/motd":
                ensure => "file",
                owner => "root",
                group => "root",
                mode => "0644",
                content => "${motd_default}\n"
        }
        file { "/root/bin":
                ensure => "directory",
                owner => "root",
                group => "root",
                mode => "0755",
        }
	file { "/root/bin/MotdWithNewerDevEnv":
                ensure => "file",
                owner => "root",
                group => "root",
                mode => "0755",
		source => "puppet:///modules/eltgeneral/MotdWithNewerDevEnv",
        }
	cron { 'CheckNewDevEnv':
		command => '/root/bin/MotdWithNewerDevEnv',
		user    => 'root',
		hour    => '2',
	}

	# ELTDEV-771. Script to upgrade a host to Beta release
	file { "/root/bin/update-puppet-beta":
                ensure => "file",
                owner => "root",
                group => "root",
                mode => "0755",
                source => "puppet:///modules/eltgeneral/update-puppet-beta",
        }

	# ELTDEV-49
	file { "/etc/NetworkManager/dispatcher.d/ifup-lo":
		ensure => "present",
		owner => "root",
		group => "root",
		mode => "0755",
		source => "puppet:///modules/eltgeneral/ifup-lo",
	}

        # ELTDEV-70:  Remove eclipse.mars if installed
        exec {  "/usr/bin/rpm -e eclipse-mars --nodeps; rm -rf /opt/eclipse/mars":
                onlyif => "/usr/bin/rpm -q --quiet eclipse-mars",
        }

	# ELTDEV-80: Add rsyslog config for ptp4l  
	file { "/etc/rsyslog.d/ptp4l.conf":
		ensure => "present",
		owner => "root",
		group => "root",
		mode => "0644",
		source => "puppet:///modules/eltgeneral/ptp4l.conf",
	}
        file_line { "eltdev-156-1":
                path => "/etc/sysctl.conf",
                line => "net.ipv4.neigh.default.gc_thresh1 = 4096",
        }
        file_line { "eltdev-156-2":
                path => "/etc/sysctl.conf",
                line => "net.ipv4.neigh.default.gc_thresh2 = 8192",
        }
        file_line { "eltdev-156-3":
                path => "/etc/sysctl.conf",
                line => "net.ipv4.neigh.default.gc_thresh3 = 8192",
        }
        file_line { "eltdev-156-4":
                path => "/etc/sysctl.conf",
                line => "net.ipv4.neigh.default.gc_stale_time = 86400",
        }

        # ELTDEV-95: create a scratch area under /home
	# It is not always possible as in NIS servers /home is automounted. 
	# It is better to check if /scratch already exists as a soft link. 
	# otherwise create /home/scratch and link /scratch -> /home/scratch
        exec { "/usr/bin/mkdir /home/scratch ; /usr/bin/chmod 1777 /home/scratch; /usr/bin/ln -s /home/scratch /":
                onlyif => "/usr/bin/test ! -h /scratch",
        }
     
	# ELTDEV-305: add kernel boot parameters to disable some performance affecting security patches
	# mitigations=off disable all optional CPU mitigations. More details can be found:
	# https://access.redhat.com/documentation/en-us/red_hat_enterprise_linux/8/html-single/8.2_release_notes/index
	$kernelargs = "mitigations=off"
	# ELTDEV-821: See lib/facter/lvm.rb
	$prefix = $facts['bootprefix']
	Package <| |> -> file_line { 'grubconfparams1':
		path	=> '/etc/default/grub',
		line	=> "GRUB_CMDLINE_LINUX=\"$bootprefix $kernelargs\"",
		match	=> "GRUB_CMDLINE_LINUX=",
		notify => Exec["grubrewrite"],
	}
	exec { "grubrewrite":
		command => "/usr/sbin/grubby --update-kernel=ALL --args=\"$kernelargs\"",
		refreshonly => "true",
	}


	# ELTDEV-28
	$puppetserver="eltint23.hq.eso.org"
	$puppetclient="${facts['hostname']}.${facts['domain']}"
	augeas { "puppetconf":
	   context => "/files/etc/puppetlabs/puppet/puppet.conf",
	   changes => [
		"set main/certname $puppetclient",
		"set main/server $puppetserver",
		"set main/environment production",
		"set main/runinterval 1h",
		],
#	    notify => Exec["puppet-running"],
	}
#
#    exec { "puppet-running":
#       command => "/opt/puppetlabs/bin/puppet resource service puppet ensure=running enable=true",
#       refreshonly => "true",
#    }

        # Default wafcache directory (and parent) for Jenkins
        Package <| |> -> file { [ "/var/cache/wafcache", "/var/cache/wafcache/global" ]:
              ensure => "directory",
              owner => "6173",  # eltdev
              group => "39542", # elt
              mode => "0755",
        }

}
